<?php

namespace Drupal\yasm\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\yasm\Services\EntitiesStatistics;
use Drupal\yasm\Utility\YasmHtml;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * YASM Statistics site contents controller.
 */
class Contents extends ControllerBase {

  /**
   * The current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entities statitistics service.
   *
   * @var \Drupal\yasm\Services\EntitiesStatistics
   */
  protected $entitiesStatistics;

  /**
   * A custom access check.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  public function access(AccountInterface $account) {
    return ($this->moduleHandler->moduleExists('node')) ? AccessResult::allowed() : AccessResult::forbidden();
  }

  /**
   * Site content page output.
   */
  public function siteContent() {
    return $this->buildContent();
  }

  /**
   * My content page output.
   */
  public function myContent() {
    // Filter contet by current user authoring.
    $build = $this->buildContent(['uid' => $this->currentUser->id()]);

    // Add user cache context because this will change for every user.
    $build['#cache']['contexts'] = ['user'];

    return $build;
  }

  /**
   * Build content page html.
   */
  private function buildContent(array $conditions = []) {
    $entity = $this->entityTypeManager->getDefinition('node');

    $info = $this->entitiesStatistics->getEntityAndBundlesInfo($entity, $conditions);
    if (isset($conditions['uid'])) {
      $this->messenger->addMessage($this->t('Statistics filtered with content authored by @user.', [
        '@user' => $this->currentUser->getDisplayname(),
      ]));
    }

    if (!empty($info['node']['bundles'])) {
      $rows = [];
      $rows[] = [
        'data' => [$this->t('Total'), $info['node']['count']],
        'class' => ['total-row'],
      ];
      foreach ($info['node']['bundles'] as $bundle) {
        $rows[] = [
          'data' => [$bundle['label'], $bundle['count']],
        ];
      }

      $table = YasmHtml::table([
        $this->t('Type'),
        $this->t('Count'),
      ], $rows);

      // Render content output.
      $build = [];
      $build[] = YasmHtml::title($this->t('Nodes'), 'far fa-file-alt');
      $build[] = $table;

      $build[] = [
        '#attached' => [
          'library' => ['yasm/global', 'yasm/fontawesome', 'yasm/datatables'],
        ],
        '#cache' => [
          'tags' => ['node_list'],
        ],
      ];

      return $build;
    }

    return ['#markup' => $this->t('No data found.')];
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(AccountInterface $current_user, EntityTypeManagerInterface $entityTypeManager, MessengerInterface $messenger, ModuleHandlerInterface $module_handler, EntitiesStatistics $entities_statistics) {
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entityTypeManager;
    $this->messenger = $messenger;
    $this->moduleHandler = $module_handler;
    $this->entitiesStatistics = $entities_statistics;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('module_handler'),
      $container->get('yasm.entities_statistics')
    );
  }

}
