<?php

namespace Drupal\yasm\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\yasm\Services\EntitiesStatistics;
use Drupal\yasm\Utility\YasmHtml;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * YASM Statistics site users controller.
 */
class Files extends ControllerBase {

  /**
   * The stream wrapper manager service.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entities statitistics service.
   *
   * @var \Drupal\yasm\Services\EntitiesStatistics
   */
  protected $entitiesStatistics;

  /**
   * A custom access check.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  public function access(AccountInterface $account) {
    return ($this->moduleHandler->moduleExists('file')) ? AccessResult::allowed() : AccessResult::forbidden();
  }

  /**
   * Site files page output.
   */
  public function siteContent() {
    return $this->buildContent();
  }

  /**
   * Build files page html.
   */
  private function buildContent() {
    $filecount = $this->entitiesStatistics->count('file');
    if ($filecount > 0) {
      $aggregates = [
        'fid'      => 'COUNT',
        'filesize' => 'SUM',
      ];

      // Count files by status.
      $files_status = $this->entitiesStatistics->aggregate('file', $aggregates, 'status');
      $status_label = [
        1 => $this->t('Permanent'),
        0 => $this->t('Temporary'),
      ];

      $rows = [];
      if (!empty($files_status)) {
        foreach ($files_status as $files) {
          $rows[] = [
            $status_label[$files['status']],
            $files['fid_count'],
            format_size($files['filesize_sum']),
          ];
        }
      }

      $table_by_status = YasmHtml::table([
        $this->t('Status'),
        $this->t('Count'),
        $this->t('Size'),
      ], $rows);

      // Count files by mimetype.
      $files_mime = $this->entitiesStatistics->aggregate('file', $aggregates, 'filemime');

      $rows = [];
      if (!empty($files_mime)) {
        foreach ($files_mime as $files) {
          $mime_parts = explode('/', $files['filemime']);
          $rows[] = [
            $this->t($mime_parts[0]),
            $mime_parts[1],
            $files['fid_count'],
            format_size($files['filesize_sum']),
          ];
        }
      }

      $table_by_mime = YasmHtml::table([
        $this->t('Type'),
        $this->t('Extension'),
        $this->t('Count'),
        $this->t('Size'),
      ], $rows);

      // Count files by stream wrapper.
      $wrappers = $this->streamWrapperManager->getWrappers();
      $rows = [];
      foreach ($wrappers as $key => $wrapper) {
        $count = $this->entitiesStatistics->aggregate('file', $aggregates, NULL, ['uri' => $key . '://%']);
        // Only add rows with content.
        if (isset($count[0]['fid_count']) && $count[0]['fid_count'] > 0) {
          $rows[] = [
            $key,
            $count[0]['fid_count'],
            format_size($count[0]['filesize_sum']),
          ];
        }
      }

      $table_by_stream = YasmHtml::table([
        $this->t('Stream wrapper'),
        $this->t('Count'),
        $this->t('Size'),
      ], $rows);

      // Build array content output.
      $build = [];
      $build[] = YasmHtml::markup($this->t('There are currently @count managed files.', [
        '@count' => $filecount,
      ]));

      $build[] = YasmHtml::title($this->t('Files by status'), 'far fa-file');
      $build[] = $table_by_status;

      $build[] = YasmHtml::title($this->t('Files by type'), 'far fa-file');
      $build[] = $table_by_mime;

      $build[] = YasmHtml::title($this->t('Files by stream wrapper'), 'far fa-file');
      $build[] = $table_by_stream;

      $build[] = [
        '#attached' => [
          'library' => ['yasm/global', 'yasm/fontawesome', 'yasm/datatables'],
        ],
        '#cache' => [
          'id' => ['yasm_files'],
          'tags' => ['file_list'],
        ],
      ];

      return $build;
    }

    return ['#markup' => $this->t('No data found.')];
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(ModuleHandlerInterface $module_handler, StreamWrapperManagerInterface $stream_wrapper_manager, EntitiesStatistics $entities_statistics) {
    $this->moduleHandler = $module_handler;
    $this->streamWrapperManager = $stream_wrapper_manager;
    $this->entitiesStatistics = $entities_statistics;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('stream_wrapper_manager'),
      $container->get('yasm.entities_statistics')
    );
  }

}
