<?php

namespace Drupal\yasm\Services;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\group\Entity\GroupInterface;

/**
 * Groups statistics class.
 */
class GroupsStatistics {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Count group contents.
   */
  public function countContents(GroupInterface $group) {
    // Use static query for performance.
    $query = $this->connection
      ->query('SELECT count(id) as count FROM {group_content_field_data} WHERE gid = :id AND type LIKE :type', [
        ':id' => $group->id(),
        ':type' => $group->getGroupType()->id() . '-group_node-%',
      ]);
    $result = $query->fetchAll();

    return $result[0]->count;
  }

  /**
   * Count group contents grouping by content type.
   */
  public function countContentsByType(GroupInterface $group) {
    // Use static query for performance.
    $query = $this->connection
      ->query('SELECT count(id) as count, type FROM {group_content_field_data} WHERE gid = :id AND type LIKE :type GROUP by type', [
        ':id' => $group->id(),
        ':type' => $group->getGroupType()->id() . '-group_node-%',
      ]);
    $results = $query->fetchAll();

    $count_types = [];
    if (!empty($results)) {
      $content_types = $this->entityTypeManager->getStorage('node')->loadMultiple();

      foreach ($results as $result) {
        if (isset($result->type)) {
          $ctype = (string) $result->type;
          $ctype = substr($ctype, strrpos($ctype, '-') + 1);
          $count_types[$ctype] = [
            'type'  => isset($content_types[$ctype]) ? $content_types[$ctype]->label() : $ctype,
            'count' => $result->count,
          ];
        }
      }
    }

    return $count_types;
  }

  /**
   * Count group members.
   */
  public function countMembers(GroupInterface $group) {
    // Use static query for performance.
    $query = $this->connection
      ->query('SELECT count(id) as count FROM {group_content_field_data} WHERE gid = :id AND type = :type', [
        ':id' => $group->id(),
        ':type' => $group->getGroupType()->id() . '-group_membership',
      ]);
    $result = $query->fetchAll();

    return $result[0]->count;
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $connection, EntityTypeManagerInterface $entityTypeManager) {
    $this->connection = $connection;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('entity_type.manager')
    );
  }

}
